@@ -185,8 +185,18 @@ class ReadWriteKafka(GatewayModule):
         default=False,
         description=("If False, replaces the timestamp field on the GatewayStruct with a timestamp autogenerated by the current Gateway."),
     )
+    include_subscribe_messages_before_engine_start: bool = Field(
+        default=False,
+        description="Based on the Kafka offset, if Kafka messages are read in with timestamps before engine start time (either the Kafka timestamp or the message on the struct), whether these messages will get dropped or included in the graph. The default is False, only pulling in messages into the csp graph that are at or after the engine starttime. If True, these messages will be pulled in with timestamp equal to engine starttime.",
+    )
     _kafkaadapter: csp.adapters.kafka.KafkaAdapterManager = PrivateAttr(default=None)
 
+    @model_validator(mode="after")
+    def check_parameters_consistent(self):
+        if self.subscribe_with_csp_engine_timestamp and not self.encoding_with_engine_timestamps:
+            raise ValueError("If `subscribe_with_csp_engine_timestamp` is True, you must set `encoding_with_engine_timestamp` to be True as well")
+        return self
+
     def __init__(self, *a, **kw):
         super().__init__(*a, **kw)
         self._kafkaadapter = csp.adapters.kafka.KafkaAdapterManager(
@@ -218,18 +228,16 @@ class ReadWriteKafka(GatewayModule):
         normalized_type = ContainerTypeNormalizer.normalize_type(ts_typ)
         is_list = get_origin(normalized_type) is list
 
-        exclusions = []
+        context = {}
         if not self.subscribe_with_struct_id:
-            exclusions.append("id")
+            context["force_new_id"] = True
         if not self.subscribe_with_struct_timestamp:
-            exclusions.append("timestamp")
+            context["force_new_timestamp"] = True
         if is_list:
             inner_type = get_args(normalized_type)[0]
-            return [inner_type.__pydantic_model__.model_validate(v).csp(exclude=exclusions) for v in obj]
-        if (pydantic_model := getattr(ts_typ, "__pydantic_model__", None)) is not None:
-            return pydantic_model.model_validate(obj).csp(exclude=exclusions)
-        else:
-            return ts_typ(**obj)
+            type_adapter = inner_type.type_adapter()
+            return [type_adapter.validate_python(v, context=context) for v in obj]
+        return ts_typ.type_adapter().validate_python(obj, context=context)
 
     def deserialize(self, encoding: str, ts_typ: object):
         """Receives the encoded string from Kafka and
@@ -310,7 +318,7 @@ class ReadWriteKafka(GatewayModule):
                     x=encoded_value,
                 )
 
-        meta_field_map = {"timestamp": "csp_timestamp"} if self.subscribe_with_csp_engine_timestamp else None
+        tick_timestamp_from_field = "csp_timestamp" if self.subscribe_with_csp_engine_timestamp else None
 
         for (
             channel_name,
@@ -324,7 +332,9 @@ class ReadWriteKafka(GatewayModule):
                     msg_mapper=msg_mapper,
                     key=key,
                     field_map=subscribe_field_map,
-                    meta_field_map=meta_field_map,
+                    tick_timestamp_from_field=tick_timestamp_from_field,
+                    adjust_out_of_order_time=True,
+                    include_msg_before_start_time=self.include_subscribe_messages_before_engine_start,
                     push_mode=csp.PushMode.NON_COLLAPSING,
                 ).encoding
                 channel_type = channels.get_outer_type(channel_name).typ
