@@ -1,5 +1,5 @@
 import typing
-from datetime import datetime
+from datetime import datetime, timezone
 
 from atomic_counter import Counter
 
@@ -11,7 +11,7 @@ def get_counter(kind: "GatewayModule"):
     if not hasattr(get_counter, "id_map"):
         get_counter.map = {}
     if kind not in get_counter.map:
-        nowish = datetime.utcnow()
+        nowish = datetime.now(timezone.utc)
         base = datetime(nowish.year, nowish.month, nowish.day)
         get_counter.map[kind] = Counter(int(base.timestamp()) * 1_000_000_000)
     return get_counter.map[kind]
